function [startPoint, stopPoint] = getShortestPathWithoutCollision(points,startID,stopID,avoidPoints1,avoidPoints2,minDistance)
    x = points(:,1);
    y = points(:,2);
    nHoles = length(x);
    
%     figure
%     plot(x,y,'*b')
%     
% 
%     hold on
%     for i=1:length(x)
%         dx = 0.1;
%         dy = 0.1;
%         text(x(i)+dx,y(i)+dy,int2str(i));
%     end
    idxs = nchoosek(1:nHoles,2);

    dist = hypot(x(idxs(:,1)) - x(idxs(:,2)),y(idxs(:,1)) - y(idxs(:,2)));
    lendist = length(dist);

    Aeq = spones(1:length(idxs)); % Adds up the number of trips
    if startID == 0 && stopID == 0
        beq = nHoles;
    else
        beq = nHoles-1;
    end

    Aeq = [Aeq;spalloc(nHoles,length(idxs),nHoles*(nHoles-1))]; % allocate a sparse matrix
    for ii = 1:nHoles
        whichIdxs = (idxs == ii); % find the trips that include stop ii
        whichIdxs = sparse(sum(whichIdxs,2)); % include trips where ii is at either end
        Aeq(ii+1,:) = whichIdxs'; % include in the constraint matrix
    end
    beq = [beq; 2*ones(nHoles,1)];
    if startID ~= 0 && stopID ~= 0
        beq(startID+1) = 1;
        beq(stopID+1) = 1;
    end
    intcon = 1:lendist;
    lb = zeros(lendist,1);
    ub = ones(lendist,1);

    opts = optimoptions('intlinprog','Display','off','Heuristics','round-diving',...
        'IPPreprocess','none');
    [x_tsp,costopt,exitflag,output] = intlinprog(dist,intcon,[],[],Aeq,beq,lb,ub,opts);

    segments = find(x_tsp); % Get indices of lines on optimal path
    %lh = zeros(nHoles,1); % Use to store handles to lines on plot
    %lh = updateSalesmanPlot(lh,x_tsp,idxs,x,y);
    %title('Solution with Subtours');
    
    collision = true;
    
    tours = detectSubtours(x_tsp,idxs);
    numtours = length(tours); % number of subtours
    fprintf('# of subtours: %d\n',numtours);

    A = spalloc(0,lendist,0); % Allocate a sparse linear inequality constraint matrix
    b = [];
    while numtours > 1 || collision % repeat until there is just one subtour and no collision
        % Add the subtour constraints
        b = [b;zeros(numtours,1)]; % allocate b
        A = [A;spalloc(numtours,lendist,nHoles)]; % a guess at how many nonzeros to allocate
        for ii = 1:numtours
            rowIdx = size(A,1)+1; % Counter for indexing
            subTourIdx = tours{ii}; % Extract the current subtour
    %         The next lines find all of the variables associated with the
    %         particular subtour, then add an inequality constraint to prohibit
    %         that subtour and all subtours that use those stops.
            variations = nchoosek(1:length(subTourIdx),2);
            for jj = 1:size(variations,1)
                whichVar = (sum(idxs==subTourIdx(variations(jj,1)),2)) & ...
                           (sum(idxs==subTourIdx(variations(jj,2)),2));
                A(rowIdx,whichVar) = 1;
            end
            b(rowIdx) = length(subTourIdx)-1; % One less trip than subtour stops
        end

        % Try to optimize again
        [x_tsp,costopt,exitflag,output] = intlinprog(dist,intcon,A,b,Aeq,beq,lb,ub,opts);

        % Visualize result
        %lh = updateSalesmanPlot(lh,x_tsp,idxs,x,y);

        % How many subtours this time?
        tours = detectSubtours(x_tsp,idxs);
        numtours = length(tours); % number of subtours
        fprintf('# of subtours: %d\n',numtours);
        if isempty(avoidPoints1) && isempty(avoidPoints2)
            collision = false;
        end
        while numtours == 1 && collision
            [collision,collision_points] = detectCollision(tours, points, avoidPoints1, avoidPoints2, minDistance);
            if collision
                fprintf('Collision detected in:\n');
                display(collision_points);
                theTour = cell2mat(tours(1,1));
                collisionTourIdxs = zeros(length(theTour)-1,2);
                for i=1:(length(theTour)-1)
                    if theTour(i) < theTour(i+1)
                        collisionTourIdxs(i,1) = theTour(i);
                        collisionTourIdxs(i,2) = theTour(i+1);
                    else
                        collisionTourIdxs(i,1) = theTour(i+1);
                        collisionTourIdxs(i,2) = theTour(i);
                    end
                end
                %whichIdxs = (idxs(:,1) == collisionTourIdxs(i,1) && idxs(:,2) == collisionTourIdxs(i,2)); % find the trips that include stop ii
                
                constraintIdxs = sparse((ismember(idxs,collisionTourIdxs,'rows'))');
                A = [A;spalloc(1,lendist,nHoles)];
                b = [b;zeros(1,1)];
                A(end,:) = constraintIdxs;
                b(end) = nHoles-2;
                
                % Try to optimize again
                [x_tsp,costopt,exitflag,output] = intlinprog(dist,intcon,A,b,Aeq,beq,lb,ub,opts);
                
                % Visualize result
                %lh = updateSalesmanPlot(lh,x_tsp,idxs,x,y);
                tours = detectSubtours(x_tsp,idxs);
                numtours = length(tours); % number of subtours
                fprintf('# of subtours: %d\n',numtours);
            else
                fprintf('No collision detected\n');
            end
        end
    end
    segments = find(round(x_tsp));
    start = zeros(length(segments),1);
    stop = start;
    for i=1:length(segments)
        start(i) = idxs(segments(i),1);
        stop(i) = idxs(segments(i),2);
    end
    theTour = cell2mat(tours(1,1));
    startPoint = zeros(length(theTour)-1,2);
    stopPoint = startPoint;
    for i=1:(length(theTour)-1)
        startPoint(i,1) = points(theTour(i),1);
        startPoint(i,2) = points(theTour(i),2);
        stopPoint(i,1) = points(theTour(i+1),1);
        stopPoint(i,2) = points(theTour(i+1),2);
    end
end