function [tunnelLine, pel, drillDir, nHoles, xStart, yStart, zStart, xEnd, yEnd, zEnd, holeType, boom, holeNoPlan, startTime, endTime, error] = readTl4Complete(filename)
    error = 0;    
    %% Open file
    fileID = fopen(filename);

    C = textscan(fileID,'%s','Delimiter','\n');
    fclose(fileID);
    C = C{1,1};
    
    %% Read data from [RoundEx]
    i = ind2sub(size(C), strmatch('[RoundEx]', C));
    tunnelLine = sscanf(sprintf('%s', C{i+1,1}), 'TunnelLine = %[^\n]');
    
    %% Read data from [Round]
    i = ind2sub(size(C), strmatch('[Round]', C));
    pel = sscanf(sprintf('%s', C{i+2,1}), 'Pel = %f');
    drillDir = sscanf(sprintf('%s', C{i+4,1}), 'DrillDir = %f');
    
    %% Read data from [Hole] and [HoleEx]
    hole = ind2sub(size(C), strmatch('[Hole]', C));
    holeEx = ind2sub(size(C), strmatch('[HoleEx]', C));
    nHoles = length(hole);
    
    xStart = zeros(nHoles,1);
    yStart = xStart;
    zStart = xStart;
    xEnd = xStart;
    yEnd = xStart;
    zEnd = xStart;
    holeType = xStart;
    boom = xStart;
    startTime = xStart;
    endTime = xStart;
    holeNoPlan = xStart;
    
    
    for j=1:nHoles
        i = hole(j);
        holeType(j) = sscanf(sprintf('%s', C{i+1,1}), 'HoleType = %d');
        boom(j) = sscanf(sprintf('%s', C{i+2,1}), 'Boom = %d');
        xStart(j) = sscanf(sprintf('%s', C{i+3,1}), 'CoordStart.X = %f');
        yStart(j) = sscanf(sprintf('%s', C{i+4,1}), 'CoordStart.Y = %f');
        zStart(j) = sscanf(sprintf('%s', C{i+5,1}), 'CoordStart.Z = %f');
        xEnd(j) = sscanf(sprintf('%s', C{i+6,1}), 'CoordEnd.X = %f');
        yEnd(j) = sscanf(sprintf('%s', C{i+7,1}), 'CoordEnd.Y = %f');
        zEnd(j) = sscanf(sprintf('%s', C{i+8,1}), 'CoordEnd.Z = %f');
        i = holeEx(j);
        holeNoPlan(j) = sscanf(sprintf('%s', C{i+2,1}), 'HolenoPlan = %d');
        if j==1
            startTime(j) = 0;
            t = sscanf(sprintf('%s', C{i+6,1}), 'Date started = %d/%d/%d %d:%d:%d');
            refTime = datetime(t.');
            
            if (refTime.Year < 1900)
                error = 1;
            end
            
            t = sscanf(sprintf('%s', C{i+7,1}), 'Date ended = %d/%d/%d %d:%d:%d');
            if (datetime(t.').Year < 1900)
                error = 1;
            end
            endTime(j) = 3600*24*datenum(datetime(t.') - refTime);
        else
            t = sscanf(sprintf('%s', C{i+6,1}), 'Date started = %d/%d/%d %d:%d:%d');
            startTime(j) = 3600*24*datenum(datetime(t.') - refTime);
            if (datetime(t.').Year < 1900)
                error = 1;
            end
            t = sscanf(sprintf('%s', C{i+7,1}), 'Date ended = %d/%d/%d %d:%d:%d');
            endTime(j) = 3600*24*datenum(datetime(t.') - refTime);
            if (datetime(t.').Year < 1900)
                error = 1;
            end
        end
    end
end