clear;
%% Read tl4 log files
%Log folder
myDir = 'C:\Users\eivindjk\Documents\Master\Hare\1110806\CL_10160 NY\2017-11-02-15-50';


tl4Files = dir(fullfile(myDir,'*SingleHole.tl4'));
if (~isempty(tl4Files))
    baseName = tl4Files(1).name;
    fullFileName = fullfile(myDir, baseName);
    fileDir = dir(fullFileName);
    if fileDir.bytes == 0

    else
        holes = readTl4(fullFileName);
    end
    for i = 2:length(tl4Files)
        baseName = tl4Files(i).name;
        fullFileName = fullfile(myDir, baseName);
        fileDir = dir(fullFileName);
        if fileDir.bytes == 0

        else
            holes(end+1) = readTl4(fullFileName);
        end
    end
end

for i=1:size(holes,1)
   if holes(i).StartTime.Year < 1900 || holes(i).EndTime.Year < 1900
       display(strcat('Error in Start/End time for hole ', tl4Files(i).name))
       return
   end
end

%% Traveling salesman
figure

first = true;
for i=1:length(holes)
    if holes(i).HoleType ~= 4 && holes(i).HoleType ~= 5 && holes(i).HoleType ~= 6
        if first
           holes_noBolts(1) = holes(i);
           first = false;
        else
            holes_noBolts(end+1) = holes(i);
        end
    end
end

% nHoles = length(holes_noBolts);
% x = zeros(nHoles,1);
% y = x;
% for i=1:nHoles
%     x(i) = holes_noBolts(i).CoordStart.Y;
%     y(i) = holes_noBolts(i).CoordStart.Z;
% end
[x,y,typeOfHole] = getDrillPlan('C:\Users\eivindjk\Documents\Bever Team\Favorites\Drillplans\STIGENDE Normalprofil 15015-15395 uten siktutvidelse med lade og tennplan.xml');
nHoles = length(x);

plot(x,y,'*b')
hold on

idxs = nchoosek(1:nHoles,2);

dist = hypot(x(idxs(:,1)) - x(idxs(:,2)),y(idxs(:,1)) - y(idxs(:,2)));
lendist = length(dist);

Aeq = spones(1:length(idxs)); % Adds up the number of trips
beq = nHoles;

Aeq = [Aeq;spalloc(nHoles,length(idxs),nHoles*(nHoles-1))]; % allocate a sparse matrix
for ii = 1:nHoles
    whichIdxs = (idxs == ii); % find the trips that include stop ii
    whichIdxs = sparse(sum(whichIdxs,2)); % include trips where ii is at either end
    Aeq(ii+1,:) = whichIdxs'; % include in the constraint matrix
end
beq = [beq; 2*ones(nHoles,1)];

intcon = 1:lendist;
lb = zeros(lendist,1);
ub = ones(lendist,1);

opts = optimoptions('intlinprog','Display','off','Heuristics','round-diving',...
    'IPPreprocess','none');
[x_tsp,costopt,exitflag,output] = intlinprog(dist,intcon,[],[],Aeq,beq,lb,ub,opts);

segments = find(x_tsp); % Get indices of lines on optimal path
lh = zeros(nHoles,1); % Use to store handles to lines on plot
lh = updateSalesmanPlot(lh,x_tsp,idxs,x,y);
title('Solution with Subtours');

tours = detectSubtours(x_tsp,idxs);
numtours = length(tours); % number of subtours
fprintf('# of subtours: %d\n',numtours);

A = spalloc(0,lendist,0); % Allocate a sparse linear inequality constraint matrix
b = [];
while numtours > 1 % repeat until there is just one subtour
    % Add the subtour constraints
    b = [b;zeros(numtours,1)]; % allocate b
    A = [A;spalloc(numtours,lendist,nHoles)]; % a guess at how many nonzeros to allocate
    for ii = 1:numtours
        rowIdx = size(A,1)+1; % Counter for indexing
        subTourIdx = tours{ii}; % Extract the current subtour
%         The next lines find all of the variables associated with the
%         particular subtour, then add an inequality constraint to prohibit
%         that subtour and all subtours that use those stops.
        variations = nchoosek(1:length(subTourIdx),2);
        for jj = 1:length(variations)
            whichVar = (sum(idxs==subTourIdx(variations(jj,1)),2)) & ...
                       (sum(idxs==subTourIdx(variations(jj,2)),2));
            A(rowIdx,whichVar) = 1;
        end
        b(rowIdx) = length(subTourIdx)-1; % One less trip than subtour stops
    end

    % Try to optimize again
    [x_tsp,costopt,exitflag,output] = intlinprog(dist,intcon,A,b,Aeq,beq,lb,ub,opts);
    
    % Visualize result
    lh = updateSalesmanPlot(lh,x_tsp,idxs,x,y);
    
    % How many subtours this time?
    tours = detectSubtours(x_tsp,idxs);
    numtours = length(tours); % number of subtours
    fprintf('# of subtours: %d\n',numtours);
end

title('Solution with Subtours Eliminated');
hold off
%%
segments = find(round(x_tsp));

splitPath = strsplit(myDir,'\');
sequenceID = char(strcat(splitPath(end-1),'\',splitPath(end)));
f = figure('Name',strcat(sequenceID,': Optimal hole sequence 2D with bolts'),'units','normalized','outerposition',[0 0 1 1],'numbertitle','off');
set(f,'Units','Pixels');
set(f, 'OuterPosition', [0, 0, 1500, 1200]);
grid on

first = true;
prev = holes(idxs(segments(1)));
hold all
scatter(x,y,32,'red');

for i=1:length(segments)
    start = idxs(segments(i),1);
    stop = idxs(segments(i),2);
    q = quiver(x(start),y(start),x(stop)-x(start),y(stop)-y(start),'AutoScale','off');
    q.Color = 'Blue';
end

drawnow

title('Optimal drill sequence 2D with bolts','FontSize',48);
xlabel('Distance [m]','FontSize',24);
ylabel('Distance [m]','FontSize',24);
set(gca,'FontSize',16)
ax = gca;
outerpos = ax.OuterPosition;
ti = ax.TightInset; 
left = outerpos(1) + ti(1);
bottom = outerpos(2) + ti(2);
ax_width = outerpos(3) - ti(1) - ti(3);
ax_height = outerpos(4) - ti(2) - ti(4);
ax.Position = [left bottom ax_width ax_height];
